
########################################
# Compile Option

COMMOM_FLAGS = -g -c -Wall -O2  -Werror \
               -MMD -MP -MF"$(@:%.o=%.d)" -MT"$@" \
               -ffunction-sections -fdata-sections \

OPTION_FLAGS = \
               -Wextra \
               -Wbuiltin-macro-redefined \
               -Wcast-align \
               -Wcast-qual \
               -Wcomment \
               -Wcomments \
               -Wconversion \
               -Wdisabled-optimization \
               -Wdouble-promotion \
               -Wendif-labels \
               -Wfloat-equal \
               -Wformat \
               -Wformat=2 \
               -Wformat-nonliteral \
               -Wformat-security \
               -Wformat-y2k \
               -Winit-self \
               -Winvalid-pch \
               -Wlogical-op \
               -Wnormalized=nfc \
               -Wmissing-declarations \
               -Wmissing-include-dirs \
               -Wmultichar \
               -Woverlength-strings \
               -Wpacked \
               -Wpedantic \
               -Wpointer-arith \
               -Wredundant-decls \
               -Wshadow \
               -Wstack-protector \
               -Wstrict-aliasing \
               -Wstrict-aliasing=3 \
               -Wstrict-overflow \
               -Wstrict-overflow=4 \
               -Wsuggest-attribute=format \
               -Wsuggest-attribute=noreturn \
               -Wswitch-default \
               -Wswitch-enum \
               -Wtrampolines \
               -Wtrigraphs \
               -Wundef \
               -Wunsafe-loop-optimizations \
               -Wunused-macros \
               -Wvector-operation-performance \
               -Wwrite-strings \
               \
               -fstrict-aliasing \

SECURE_FLAGS = \
               -fstack-protector-all \
               -ftrapv  \
               -D_FORTIFY_SOURCE=2 \

## unused for c
EXTRA_FLAGS =  \

C_OPTION_FLAG = \
               -Wbad-function-cast \
               -Wc++-compat \
               -Wdeclaration-after-statement \
               -Wimplicit \
               -Wimplicit-function-declaration \
               -Wimplicit-int \
               -Wjump-misses-init \
               -Wmissing-parameter-type \
               -Wmissing-prototypes \
               -Wnested-externs \
               -Wold-style-declaration \
               -Wold-style-definition \
               -Woverride-init \
               -Wpointer-sign \
               -Wpointer-to-int-cast \
               -Wstrict-prototypes \
               -Wtraditional \
               -Wtraditional-conversion \
               -Wunsuffixed-float-constants \

## unused
C_EXTRA_FLAGS =  \

CPP_OPTION_FLAG = \
               -Wc++0x-compat \
               -Wc++11-compat \
               -Wctor-dtor-privacy \
               -Wdelete-non-virtual-dtor \
               -Weffc++ \
               -Wliteral-suffix \
               -Wnoexcept \
               -Wnon-virtual-dtor \
               -Wold-style-cast \
               -Woverloaded-virtual \
               -Wreorder \
               -Wsign-promo \
               -Wstrict-null-sentinel \
               -Wsynth \
               -Wuseless-cast \
               -Wzero-as-null-pointer-constant \
               \
               -fthreadsafe-statics \

## not support armgcc
ARMGCC_NOT_SUPPORT = \

## unused
CPP_EXTRA_FLAGS = \

LIBFLAGS        = -rD

ifeq ($(ARCH), ppc)
CFLAGS          = $(COMMOM_FLAGS) $(OPTION_FLAGS) $(EXTRA_FLAGS) $(C_OPTION_FLAG)
CCFLAGS         = $(COMMOM_FLAGS) $(OPTION_FLAGS) $(EXTRA_FLAGS) $(CPP_OPTION_FLAG)
endif

ifeq ($(ARCH), ppc_fsl)
FSL_FLAGS       = -m32 -mcpu=8548 -mabi=spe -mspe -mfloat-gprs=double --sysroot=$(LIBROOT) \
                  -pipe -feliminate-unused-debug-types
CFLAGS          = $(COMMOM_FLAGS) $(OPTION_FLAGS) $(EXTRA_FLAGS) $(C_OPTION_FLAG) $(FSL_FLAGS) \
CCFLAGS         = $(COMMOM_FLAGS) $(OPTION_FLAGS) $(EXTRA_FLAGS) $(CPP_OPTION_FLAG) $(FSL_FLAGS) \
                 -std=c++11
endif

ifeq ($(ARCH), arm)
CFLAGS         = $(COMMOM_FLAGS) $(OPTION_FLAGS) $(C_OPTION_FLAG)
CCFLAGS        = -std=c++11 $(COMMOM_FLAGS) $(OPTION_FLAGS) $(EXTRA_FLAGS) $(CPP_OPTION_FLAG)
endif

ifeq ($(ARCH), x86)
COMMOM_FLAGS   += -DX86_GCC -D__X86__ $(SECURE_FLAGS)
CFLAGS         = $(COMMOM_FLAGS) $(OPTION_FLAGS) $(C_OPTION_FLAG)
CCFLAGS        = $(COMMOM_FLAGS) $(OPTION_FLAGS) $(EXTRA_FLAGS) $(CPP_OPTION_FLAG) \
                 -std=c++11
endif

MAKEDEPEND_FLAGS = -m


########################################
# Compile Rule
#
.SUFFIXES:

$(OBJ_DIR)/%.o: $(SRC_DIR)/%.c
	$(CC) $(CFLAGS) $(CCMACRO) $(INC_DIR) -c $< -o $@

$(OBJ_DIR)/%.o: $(SRC_DIR)/%.cpp
	$(CPP) $(CCFLAGS) $(CCMACRO) $(INC_DIR) -c $< -o $@

$(OBJ_DIR)/%.o: $(SRC_DIR)/%.cc
	$(CPP) $(CCFLAGS) $(CCMACRO) $(INC_DIR) -c $< -o $@

%.o: %.c
	$(CC) $(CFLAGS) $(CCMACRO) $(INC_DIR) -c $< -o $@

%.o: %.cpp
	$(CPP) $(CCFLAGS) $(CCMACRO) $(INC_DIR) -c $< -o $@

%.o: %.cc
	$(CPP) $(CCFLAGS) $(CCMACRO) $(INC_DIR) -c $< -o $@



$(OBJ_DIR)/%.d: $(SRC_DIR)/%.c
	@if [ ! -d $(dir $@) ]; then mkdir -p $(dir $@); fi
	@(($(MAKEDEPEND) $(INC_DIR) $(MAKED_INC) $(MAKEDEPEND_FLAGS) -p $(OBJ_DIR)/ -f - $<  | \
	sed 's|$(OBJ_DIR)\/$(SRC_DIR)|$(OBJ_DIR)|g' > $@) 2>&1) > /dev/null

%.d: %.c
	@if [ ! -d $(dir $@) ]; then mkdir -p $(dir $@); fi
	@(($(MAKEDEPEND) $(INC_DIR) $(MAKED_INC) $(MAKEDEPEND_FLAGS) -p $(OBJ_DIR)/ -f - $<  | \
	sed -e 's!$*\.o!& $@!g' > $@) 2>&1) > /dev/null

$(OBJ_DIR)/%.d: $(SRC_DIR)/%.cpp
	@if [ ! -d $(dir $@) ]; then mkdir -p $(dir $@); fi
	@(($(MAKEDEPEND) $(INC_DIR) $(MAKED_INC) $(MAKEDEPEND_FLAGS) -p $(OBJ_DIR)/ -f - $<  | \
	sed 's|$(OBJ_DIR)\/$(SRC_DIR)|$(OBJ_DIR)|g' > $@) 2>&1) > /dev/null

%.d: %.cpp
	@if [ ! -d $(dir $@) ]; then mkdir -p $(dir $@); fi
	@(($(MAKEDEPEND) $(INC_DIR) $(MAKED_INC) $(MAKEDEPEND_FLAGS) -p $(OBJ_DIR)/ -f - $<  | \
	sed -e 's!$*\.o!& $@!g' > $@) 2>&1) > /dev/null

$(OBJ_DIR)/%.d: $(SRC_DIR)/%.cc
	@if [ ! -d $(dir $@) ]; then mkdir -p $(dir $@); fi
	@(($(MAKEDEPEND) $(INC_DIR) $(MAKED_INC) $(MAKEDEPEND_FLAGS) -p $(OBJ_DIR)/ -f - $<  | \
	sed 's|$(OBJ_DIR)\/$(SRC_DIR)|$(OBJ_DIR)|g' > $@) 2>&1) > /dev/null

%.d: %.cc
	@if [ ! -d $(dir $@) ]; then mkdir -p $(dir $@); fi
	@(($(MAKEDEPEND) $(INC_DIR) $(MAKED_INC) $(MAKEDEPEND_FLAGS) -p $(OBJ_DIR)/ -f - $<  | \
	sed -e 's!$*\.o!& $@!g' > $@) 2>&1) > /dev/null


ifneq ($(MAKECMDGOALS), depend)
ifneq ($(MAKECMDGOALS), clean)
-include $(DPDS)
endif
endif
