########################################
#
# Copyright (c) 2015 Sony Corporation
#
########################################

GCC_VER = $(shell g++ --version | grep GCC | sed 's/^g++ (GCC) //'| sed 's/ .*//' | \
            sed 's/\([0-9]*\)\.\([0-9]*\)\.\([0-9]*\)/\1 \2 \3/' |  \
            xargs printf "%02d%02d%02d")

GCC_VER_MIN = 040801

LIBS     = $(LIBGTEST) $(LIBGMOCK) -lz -lzstd -lpthread -lgtest -lgtest_main -lgmock $(EXTLIBS)

INC_DIR += \
        -I$(PRJ_BASE)/$(GTEST_DIR)/include \
        -I$(PRJ_BASE)/$(GMOCK_DIR)/include \
        -I$(PRJ_BASE)/include

CCFLAGS := -g -O0 -std=gnu++0x -DX86_GCC -DUNIT_TEST -Werror -Wall -coverage

GCOVS    = *.gcda *.gcno *.gcov

SRC := $(notdir $(SRC))

.PHONY: gtest
gtest: test
	@./test $(GOPT); \
	if [ ! $$? = 0 ]; then exit 1; fi; \
	make gcov;

.PHONY: test
test: check_version $(OBJS) $(LIBOBJS)
	@$(CPP) $(CCFLAGS) $(OBJS) $(LIBOBJS) $(LIBS) -o $@

.PHONY: gext
gext: ext_test
	@./ext_test $(GOPT); \
	if [ ! $$? = 0 ]; then exit 1; fi; \
	make gcov;

.PHONY: ext_test
ext_test: check_version $(EXT_OBJS) $(LIBOBJS)
	@$(CPP) $(CCFLAGS) $(EXT_OBJS) $(LIBOBJS) $(LIBS) -o $@

.PHONY: check_version
check_version:
	@echo -e "$(COL_CYAN) -----> check gcc version $(COL_E)"; \
	if [ $(GCC_VER) -gt $(GCC_VER_MIN) ]; then \
	    echo -e "$(COL_GREEN) version : $(GCC_VER) $(COL_E)"; \
	else \
	    echo -e "$(COL_RED) ERROR :  $(GCC_VER) < $(GCC_VER_MIN) $(COL_E)"; \
	    exit -1; \
	fi

.PHONY: clean
clean:
	@$(RM) $(OBJS) $(EXT_OBJS) $(DPDS) $(GCOVS) test ext_test;

.PHONY: clean_test_gcda
clean_test_gcda :
	$(RM) -rf test*.gcda

.PHONY: gcov
gcov :
	@echo -e "$(COL_CYAN) -----> creating gcov $(COL_E)"; \
	for i in $(SRC); do \
	(file=`echo $$i | sed -e 's|cpp|gcda|g'`; \
		if [ -f $$file ]; then \
		(echo -e "$(COL_CYAN) --> gcov in $$file ... $(COL_E)" && \
			gcov -r -c $$file;) || exit 1; \
		fi); \
	done;

.PHONY: depend
depend: $(DPDS)
