########################################
# Extract Warning Message
#

define func_title
	echo ;\
	echo -e "$(COL_MAGENTA) ======================================= $(COL_E)"; \
	echo -e "$(COL_YELLOW)  $1 $(COL_E)"; \
	echo -e "$(COL_MAGENTA) ======================================= $(COL_E)";
endef

define func_msg
	echo ;\
	echo -e "$(COL_MAGENTA) ---------------------------------------- $(COL_E)"; \
	echo -e "$(COL_YELLOW) -----> Target : [ $1 ] $2 $(COL_E)"; \
	echo -e "$(COL_MAGENTA) ---------------------------------------- $(COL_E)";
endef

.PHONY: warning
warning:
	@make warning_log

.PHONY: warning_log
warning_log:
	@(warning_lst=`                             \
		grep -e ": warning" -e": ٹ" $(MAKELOG)| \
		sed 's|^.*\/||g' |                      \
		sort -u`;                               \
	if [ "$$warning_lst" != "" ]; then          \
		echo -n "******* Warning List *******"; \
		echo ;                                  \
		echo "$$warning_lst" ;                  \
		echo -n "******* See " ;                \
		echo -n $(MAKELOG);                     \
		echo " for detail ********";            \
		echo "$$warning_lst" > $(WARNINGLOG);   \
		num_of_warning=`wc -l $(WARNINGLOG) |   \
		sed 's|[^0-9]*||g'`;                    \
		echo -n "warning : "  >> $(WARNINGLOG); \
		echo $$num_of_warning >> $(WARNINGLOG); \
	fi)

STIME=START_TIME
.PHONY: prepare
prepare:
	@date +%s > $(STIME); \
	rm -rf $(TARGET)

.PHONY: compile_check
compile_check:
	@echo -e "$(COL_CYAN) -----> compile check $(COL_E)"; \
	if [ -f "$(TARGET)" ]; then                                     \
		(if [ -f "$(STIME)" ]; then                                 \
		    cur_time=`date +%s`;                                    \
		    start_time=`cat $(STIME)`;                              \
		    past_time=`expr $$cur_time - $$start_time`;             \
		    echo -e "$(COL_GREEN) OK [$$past_time sec] $(COL_E)";   \
		    rm -rf $(STIME);                                        \
		 else                                                       \
		    echo -e "$(COL_GREEN) COMPILE OK $(COL_E)";             \
		 fi;                                                        \
		)                                                           \
	else                                                            \
		echo -e "$(COL_RED) TARGET not found $(COL_E)";             \
		exit 1;                                                     \
	fi

CPP_EXT="c,cc,h,hpp,c++,h++,hh,cu,cpp,hxx,cxx,cuh"

ifndef CPPLINT_LINE_LENGTH
    CPPLINT_LINE_LENGTH="80"
endif

ifndef CPPLINT_FILTERS
    CPPLINT_FILTERS="-build/include,\
                     -build/c++11,\
                     -build/header_guard,\
                     -legal/copyright,\
                     -readability/braces,\
                     -whitespace/indent,\
                     -whitespace/newline"
endif


.PHONY: check_style
check_style:
	@echo -e "$(COL_CYAN) -----> check style (cpp)$(COL_E)";    \
	$(CPPLINT) --linelength=$(CPPLINT_LINE_LENGTH)              \
	           --root=.                                         \
	           --extensions=$(CPP_EXT)                          \
	           --filter=$(CPPLINT_FILTERS)                      \
	           --exclude={$(CPPLINT_EXCLUDE_PATH),}             \
	           --recursive . ;                                  \
	if [ $$? = 0 ]; then                                        \
		echo -e "$(COL_GREEN)CHECK OK $(COL_E)";                \
		exit 0;                                                 \
	else                                                        \
		echo -e "$(COL_RED)CHECK NG $(COL_E)";                  \
		exit 1;                                                 \
	fi


.PHONY: check_dep
check_dep:
	@( 																			\
		num_of_inc=`find $(OBJ_DIR) -name "*.d"|xargs cat |grep "\.h:"|wc -l`; 	\
		num_of_dep=`find $(OBJ_DIR) -name "*.d"|wc -l`; 						\
		tako=`expr $$num_of_inc / $$num_of_dep`; 								\
		echo "num of depend file   : $$num_of_dep";   							\
		echo "num of include header: $$num_of_inc";   							\
		echo "header / file        : $$tako"; 									\
	)

.PHONY: check_line
check_line:
	@( 																			\
		find . -name '*.c' -or -name '*.cpp' -or -name '*.h' |xargs cat|wc --line \
	)

.PHONY: check_file
check_file:
	@! find -path ./deps -prune -o \( -iname "*.[ch]" -o -iname "*.[ch]pp" \) -not -type l \
	| xargs nkf -g | grep -v "UTF-8\|ASCII"
	@echo "*** char_code : OK ***"

.PHONY: check_file_all
check_file_all:
	@$(call func_title,"Check File All")
	@! find                             \( -iname "*.[ch]" -o -iname "*.[ch]pp" \) -not -type l \
	| xargs nkf -g | grep -v "UTF-8\|ASCII"
	@echo "*** char_code : OK ***"

STM_NG_LIST="gets scanf sprintf strcat strcpy vsprintf wcscat wcscpy system popen"
.PHONY: stm
stm:
	@( \
	is_detect=0; \
	for word in "$(STM_NG_LIST)" ; \
	do \
		echo "[ $$word ]"; \
		find . -name "*.[ch]" -o -name "*.cpp" | xargs grep -w -n "$$word" /dev/null | grep -E "$$word\s*\("; \
		if [ $$? -eq 0 ]; then is_detect=1; fi; \
	done; \
	if [ $$is_detect -eq 1 ]; then \
		echo -e "$(COL_RED)STM ERROR $(COL_E)"; \
		exit 1; \
	else \
		echo -e "$(COL_GREEN)STM OK $(COL_E)"; \
		exit 0; \
	fi; \
	)

.PHONY: ccclean
ccclean:
	@ccache -C

.PHONY: bug
bug :
	@echo -e "\n\n" ; \
	 echo -e "$(COL_RED)                             oooo############oooo                                 $(COL_E)" ; \
	 echo -e "$(COL_RED)                         oo########################o                              $(COL_E)" ; \
	 echo -e "$(COL_RED)                      oo##############################o         o#   ## o#        $(COL_E)" ; \
	 echo -e "$(COL_RED)      o # oo        o####################################o       ## ## ##o#       $(COL_E)" ; \
	 echo -e "$(COL_RED)   oo # # '#      o#########    #############    #########o       ###o##o#        $(COL_E)" ; \
	 echo -e "$(COL_RED)   '######o#     o#########      ###########      ##########o    ########         $(COL_E)" ; \
	 echo -e "$(COL_RED)     #######    ###########      ###########      #######################         $(COL_E)" ; \
	 echo -e "$(COL_RED)     #######################    #############    ##############  '''###           $(COL_E)" ; \
	 echo -e "$(COL_RED)      '###''''#################################################     '###          $(COL_E)" ; \
	 echo -e "$(COL_RED)       ###   o##################################################     '###o        $(COL_E)" ; \
	 echo -e "$(COL_RED)      o##'   ###################################################       ###o       $(COL_E)" ; \
	 echo -e "$(COL_RED)      ###    #############################################' '######ooooo####o     $(COL_E)" ; \
	 echo -e "$(COL_RED)     o###oooo#####  #####################################   o#################    $(COL_E)" ; \
	 echo -e "$(COL_RED)     ########'####   ##################################     ####''''''''          $(COL_E)" ; \
	 echo -e "$(COL_RED)    ''''       ####    '############################'      o###                   $(COL_E)" ; \
	 echo -e "$(COL_RED)               '###o     '''##################'##'         ###                    $(COL_E)" ; \
	 echo -e "$(COL_RED)                 ###o          '##''######''''           o###                     $(COL_E)" ; \
	 echo -e "$(COL_RED)                  ####o                 oo             o###'                      $(COL_E)" ; \
	 echo -e "$(COL_RED)                   '####o      o######o'####o        o####                        $(COL_E)" ; \
	 echo -e "$(COL_RED)                     '#####oo     ''####o#####o   o####''                         $(COL_E)" ; \
	 echo -e "$(COL_RED)                        ''#####oooo  '###o#########'''                            $(COL_E)" ; \
	 echo -e "$(COL_RED)                           ''#######oo ##########                                 $(COL_E)" ; \
	 echo -e "$(COL_RED)                                   ''''###########                                $(COL_E)" ; \
	 echo -e "$(COL_RED)    dMMMMb   dMP dMP  .aMMMMP  dM dM   ############                               $(COL_E)" ; \
	 echo -e "$(COL_RED)   dMP'dMP  dMP dMP  dMP'     dM dM     ##########'                               $(COL_E)" ; \
	 echo -e "$(COL_RED)  dMMMMK'  dMP dMP  dMP MMP' dM dM       '###''''                                 $(COL_E)" ; \
	 echo -e "$(COL_RED) dMP.aMF  dMP.aMP  dMP.dMP  '' ''                                                 $(COL_E)" ; \
	 echo -e "$(COL_RED) MMMMP'   VMMMP'   VMMMP'  dM dM                                                  $(COL_E)" ; \
	 echo -e "\n\n" ; \

.PHONY: bug2
bug2 :
	@echo -e "\n\n" ; \
	 echo -e "$(COL_RED)             \\    o ^ o    \"             $(COL_E)" ; \
	 echo -e "$(COL_RED)               \\ (     ) \"               $(COL_E)" ; \
	 echo -e "$(COL_RED)   ________ ___(%%%%%%%%%)____________     $(COL_E)" ; \
	 echo -e "$(COL_RED)  (    /   /  (%%%%%%%%%%%) \\   \\    )   $(COL_E)" ; \
	 echo -e "$(COL_RED)  (___/___/__ (           )\\__\\___\\___) $(COL_E)" ; \
	 echo -e "$(COL_RED)     (    /  /(%%%%%%%%%%%) \\  \\   )     $(COL_E)" ; \
	 echo -e "$(COL_RED)      (__/__/_ (%%%%%%%%%)___\\__\\__)     $(COL_E)" ; \
	 echo -e "$(COL_RED)              /(         )\\               $(COL_E)" ; \
	 echo -e "$(COL_RED)             /  (%%%%%%%)   \\             $(COL_E)" ; \
	 echo -e "$(COL_RED)                 (%%%%%)                   $(COL_E)" ; \
	 echo -e "$(COL_RED)                    !                      $(COL_E)" ; \
	 echo -e "\n\n" ; \

.PHONY: complete
complete :
	@echo -e "\n\n" ; \
	 echo -e "$(COL_GREEN)    .aMMMb   .aMMMb   dMMMMMMMMb   dMMMMb  dMP    dMMMMMP dMMMMMMP dMMMMMP $(COL_E)" ; \
	 echo -e "$(COL_GREEN)   dMP'VMP  dMP'dMP  dMP'dMP'dMP  dMP.dMP dMP    dMP       dMP    dMP      $(COL_E)" ; \
	 echo -e "$(COL_GREEN)  dMP      dMP dMP  dMP dMP dMP  dMMMMP' dMP    dMMMP     dMP    dMMMP     $(COL_E)" ; \
	 echo -e "$(COL_GREEN) dMP.aMP  dMP.aMP  dMP dMP dMP  dMP    dMP     dMP       dMP    dMP        $(COL_E)" ; \
	 echo -e "$(COL_GREEN) VMMMP'   VMMMP'  dMP dMP dMP  dMP    dMMMMMP dMMMMMP   dMP    dMMMMMP     $(COL_E)" ; \
	 echo -e "\n\n" ; \

.PHONY: good
good :
	@echo -e "\n\n" ; \
	 echo -e "$(COL_GREEN)    .aMMMMP   .aMMMb   .aMMMb    dMMMMb    dM dM $(COL_E)" ; \
	 echo -e "$(COL_GREEN)   dMP'      dMP'dMP  dMP'dMP   dMP VMP   dM dM  $(COL_E)" ; \
	 echo -e "$(COL_GREEN)  dMP MMP'  dMP dMP  dMP dMP   dMP dMP   dM dM   $(COL_E)" ; \
	 echo -e "$(COL_GREEN) dMP.dMP   dMP.aMP  dMP.aMP   dMP.aMP   '' ''    $(COL_E)" ; \
	 echo -e "$(COL_GREEN) VMMMP'    VMMMP'   VMMMP'   dMMMMP'  dM dM      $(COL_E)" ; \
	 echo -e "\n\n" ; \

.PHONY: error
error :
	@echo -e "\n\n" ; \
	echo -e "$(COL_RED)     dMMMMMP   dMMMMb    dMMMMb   .aMMMb    dMMMMb $(COL_E)" ; \
	echo -e "$(COL_RED)    dMP       dMP.dMP   dMP.dMP  dMP'dMP   dMP.dMP $(COL_E)" ; \
	echo -e "$(COL_RED)   dMMMP     dMMMMK'   dMMMMK'  dMP dMP   dMMMMK'  $(COL_E)" ; \
	echo -e "$(COL_RED)  dMP       dMP'AMF   dMP'AMF  dMP.aMP   dMP'AMF   $(COL_E)" ; \
	echo -e "$(COL_RED) dMMMMMP   dMP dMP   dMP dMP   VMMMP'   dMP dMP    $(COL_E)" ; \
	echo -e "\n\n" ; \

.PHONY: alphabet
alphabet :
	@echo -e "\n\n" ; \
	 echo -e "$(COL_GREEN)       .aMMMb   dMMMMb   .aMMMb   dMMMMb   dMMMMMP  dMMMMMP  .aMMMMP   $(COL_E)" ; \
	 echo -e "$(COL_GREEN)      dMP'dMP  dMP'dMP  dMP'VMP  dMP VMP  dMP      dMP      dMP'       $(COL_E)" ; \
	 echo -e "$(COL_GREEN)     dMMMMMP  dMMMMK'  dMP      dMP dMP  dMMMP    dMMMP    dMP MMP'    $(COL_E)" ; \
	 echo -e "$(COL_GREEN)    dMP dMP  dMP.aMF  dMP.aMP  dMP.aMP  dMP      dMP      dMP.dMP      $(COL_E)" ; \
	 echo -e "$(COL_GREEN)   dMP dMP  dMMMMP'   VMMMP'  dMMMMP'  dMMMMMP  dMP       VMMMP'       $(COL_E)" ; \
	 echo -e "$(COL_GREEN)                                                                       $(COL_E)" ; \
	 echo -e "$(COL_GREEN)       dMP dMP  dMP  dMMMMMP  dMP dMP  dMP      dMMMMMMMMb   dMMMMb    $(COL_E)" ; \
	 echo -e "$(COL_GREEN)      dMP dMP  amr      dMP  dMP.dMP  dMP      dMP'dMP'dMP  dMP dMP    $(COL_E)" ; \
	 echo -e "$(COL_GREEN)     dMMMMMP  dMP      dMP  dMMMMK'  dMP      dMP dMP dMP  dMP dMP     $(COL_E)" ; \
	 echo -e "$(COL_GREEN)    dMP dMP  dMP  dK .dMP  dMP'AMF  dMP      dMP dMP dMP  dMP dMP      $(COL_E)" ; \
	 echo -e "$(COL_GREEN)   dMP dMP  dMP   VMMMP'  dMP dMP  dMMMMMP  dMP dMP dMP  dMP dMP       $(COL_E)" ; \
	 echo -e "$(COL_GREEN)                                                                       $(COL_E)" ; \
	 echo -e "$(COL_GREEN)       .aMMMb   dMMMMb   .aMMMb   dMMMMb   .dMMMb  dMMMMMMP  dMP dMP   $(COL_E)" ; \
	 echo -e "$(COL_GREEN)      dMP'dMP  dMP.dMP  dMP'dMP  dMP.dMP  dMP' VP    dMP    dMP dMP    $(COL_E)" ; \
	 echo -e "$(COL_GREEN)     dMP dMP  dMMMMP'  dMP.dMP  dMMMMK'   VMMMb     dMP    dMP dMP     $(COL_E)" ; \
	 echo -e "$(COL_GREEN)    dMP.aMP  dMP      dMP.MMP  dMP'AMF  dP .dMP    dMP    dMP.aMP      $(COL_E)" ; \
	 echo -e "$(COL_GREEN)    VMMMP'  dMP       VMMP'MP dMP dMP   VMMMP'    dMP     VMMMP'       $(COL_E)" ; \
	 echo -e "$(COL_GREEN)                                                                       $(COL_E)" ; \
	 echo -e "$(COL_GREEN)       dMP dMP  dMP dMP dMP  dMP dMP  dMP dMP  dMMMMMP                 $(COL_E)" ; \
	 echo -e "$(COL_GREEN)      dMP dMP  dMP dMP dMP  dMK.dMP  dMP.dMP    .dMP'                  $(COL_E)" ; \
	 echo -e "$(COL_GREEN)     dMP dMP  dMP dMP dMP  .dMMMK'   VMMMMP   .dMP'                    $(COL_E)" ; \
	 echo -e "$(COL_GREEN)     YMvAP'  dMP.dMP.dMP  dMP'AMF  dA .dMP  .dMP'    amr               $(COL_E)" ; \
	 echo -e "$(COL_GREEN)      VP'    VMMMPVMMP'  dMP dMP   VMMMP'  dMMMMMP  dMP                $(COL_E)" ; \
	 echo -e "\n\n" ; \

VALGLIND_EXEC="valgrind.out"
VALGLIND_LOG="valgrind.log"
LEAK_STR="definitely lost: [1-9]"
INVALID_ACCESS_STR="Invalid write of size"
.PHONY: valgrind
valgrind: $(OBJS) $(LIBOBJS)
	@$(CPP) $(CCFLAGS) $(OBJS) $(LIBOBJS) $(LIBS) -o $(VALGLIND_EXEC) ;                                                        \
		valgrind --tool=memcheck --leak-check=full --log-file=$(VALGLIND_LOG) ./$(VALGLIND_EXEC) > /dev/null 2>&1 || true ;    \
		LEAK_ER=`grep -cE $(LEAK_STR) $(VALGLIND_LOG)` ;                                                                        \
		MEM_ER=`grep -c $(INVALID_ACCESS_STR) $(VALGLIND_LOG)` ;                                                               \
		if [ 0 -ne $${MEM_ER} ]; then  echo -e "$(COL_RED)[ERROR] Invalid Access!!!$(COL_E)"; fi;                              \
		if [ 0 -ne $${LEAK_ER} ]; then echo -e "$(COL_RED)[ERROR] Memory Leak!!!$(COL_E)"; fi;                                 \
		if [ 0 -ne `expr $${LEAK_ER} + $${MEM_ER}` ]; then echo See `pwd`/$(VALGLIND_LOG) ; exit 1; else echo "[[ OK ]]" ; fi;
	@echo -e "\n"

.PHONY: clean_valgrind
clean_valgrind:
	@$(RM) $(VALGLIND_EXEC) $(VALGLIND_LOG)
