#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from ioctl_opt import IOW, IOR, IOWR, IO
import ctypes
class video_dmac_region(ctypes.Structure):
    _fields_ = [
        ('gpu_offset', ctypes.c_uint),
        ('size', ctypes.c_uint),
        ('handle', ctypes.c_uint),
    ]

class video_dmac_descriptor(ctypes.Structure):
    _fields_ = [
        ('port', ctypes.c_uint),
        ('handle', ctypes.c_uint),
        ('ch', ctypes.c_uint),
        ('strip', ctypes.c_uint),
        ('direction', ctypes.c_uint),
        ('kind', ctypes.c_uint),
    ]

class video_dmac_channel_setting(ctypes.Structure):
    _fields_ = [
        ('ch', ctypes.c_uint),
        ('write_port', ctypes.c_uint),
        ('read_port', ctypes.c_uint),
    ]

class gpu_mem_info(ctypes.Structure):
    _fields_ = [
        ('address', ctypes.c_void_p),
        ('size', ctypes.c_uint),
    ]

class fpga_reg_write_arg(ctypes.Structure):
    _fields_ = [
        ('dst_address', ctypes.c_uint),
        ('reg_top', ctypes.POINTER(ctypes.c_uint)),
        ('size', ctypes.c_uint),
    ]

class multicast_info(ctypes.Structure):
    _fields_ = [
        ('number', ctypes.c_uint),
        ('address', ctypes.c_ulong),
    ]

class ctrl_dmac_ignore_list(ctypes.Structure):
    _fields_ = [
        ('list_addr', ctypes.POINTER(ctypes.c_uint)),
        ('size', ctypes.c_uint),
    ]

class video_file_info(ctypes.Structure):
    _fields_ = [
        ('format', ctypes.c_uint),
        ('dst_address', ctypes.c_ulong),
        ('buffer_address', ctypes.POINTER(ctypes.c_uint)),
        ('buffer_size', ctypes.c_uint),
    ]

class audio_header(ctypes.Structure):
    _fields_ = [
        ('sample_status', ctypes.c_ubyte),
        ('endian', ctypes.c_ubyte),
        ('ch_assign', ctypes.c_ushort),
        ('payload_size', ctypes.c_uint),
        ('sample_size', ctypes.c_uint),
    ]

class audio_file_info(ctypes.Structure):
    _fields_ = [
        ('head', audio_header),
        ('target_addr', ctypes.c_ulong),
        ('next_read', ctypes.c_uint),
        ('buffer_address', ctypes.POINTER(ctypes.c_uint)),
        ('buffer_size', ctypes.c_uint),
    ]

class slavedata_write_info(ctypes.Structure):
    _fields_ = [
        ('block_num', ctypes.c_uint),
        ('buffer_address', ctypes.POINTER(ctypes.c_uint)),
    ]

class slavedata_start_param(ctypes.Structure):
    _fields_ = [
        ('file_target', ctypes.c_uint),
        ('block_count', ctypes.c_uint),
    ]

SONY_FPGA_IOC_MAGIC = (0x73)
SONY_FPGA_PIN_GPU_MEMORY = IOW(SONY_FPGA_IOC_MAGIC, 1, gpu_mem_info)
SONY_FPGA_WAIT_VD_INFO = IOR(SONY_FPGA_IOC_MAGIC, 2, ctypes.c_uint)
SONY_FPGA_VD_INT_ENABLE = IOW(SONY_FPGA_IOC_MAGIC, 3, ctypes.c_uint)
SONY_FPGA_REG_WRITE = IOW(SONY_FPGA_IOC_MAGIC, 4, fpga_reg_write_arg)
SONY_FPGA_REG_UPDATE_COMPLETED = IO(SONY_FPGA_IOC_MAGIC, 5)
SONY_FPGA_REG_UPDATE_OK = IO(SONY_FPGA_IOC_MAGIC, 6)
SONY_FPGA_MAKE_VIDEO_DESCRIPTOR = IOWR(SONY_FPGA_IOC_MAGIC, 7, video_dmac_region)
SONY_FPGA_RESET_VIDEO_DESCRIPTOR = IO(SONY_FPGA_IOC_MAGIC, 8)
SONY_FPGA_SET_VIDEO_DESCRIPTOR = IOW(SONY_FPGA_IOC_MAGIC, 9, video_dmac_descriptor)
SONY_FPGA_SET_VIDEO_DMAC = IOW(SONY_FPGA_IOC_MAGIC, 10, ctypes.c_uint)
SONY_FPGA_SET_MASTER_MODE = IOW(SONY_FPGA_IOC_MAGIC, 11, ctypes.c_uint)
SONY_FPGA_SET_CTRL_DMAC = IOW(SONY_FPGA_IOC_MAGIC, 12, ctypes.c_uint)
SONY_FPGA_SET_MULTICAST = IOW(SONY_FPGA_IOC_MAGIC, 13, ctypes.c_uint)
SONY_FPGA_SET_MULTICAST_ADDRESS = IOW(SONY_FPGA_IOC_MAGIC, 14, multicast_info)
SONY_FPGA_CTRL_DMAC_IGNORE_ADDRESS = IOW(SONY_FPGA_IOC_MAGIC, 15, ctrl_dmac_ignore_list)
SONY_FPGA_CTRL_DMAC_IGNORE_LIST_CLEAR = IO(SONY_FPGA_IOC_MAGIC, 16)
SONY_FPGA_GET_DMAC_FILE_STATUS = IOR(SONY_FPGA_IOC_MAGIC, 17, ctypes.c_uint)
SONY_FPGA_VIDEO_FILE_WRITE = IOWR(SONY_FPGA_IOC_MAGIC, 18, video_file_info)
SONY_FPGA_VIDEO_FILE_READ = IOWR(SONY_FPGA_IOC_MAGIC, 19, video_file_info)
SONY_FPGA_SET_SDI_OUT_CH = IOW(SONY_FPGA_IOC_MAGIC, 20, ctypes.c_uint)
SONY_FPGA_REF_LOCK = IO(SONY_FPGA_IOC_MAGIC, 21)
SONY_FPGA_AUDIO_MUTE = IOW(SONY_FPGA_IOC_MAGIC, 22, ctypes.c_uint)
SONY_FPGA_AUDIO_FILE_WRITE = IOWR(SONY_FPGA_IOC_MAGIC, 23, audio_file_info)
SONY_FPGA_AUDIO_FILE_READ = IOWR(SONY_FPGA_IOC_MAGIC, 24, audio_file_info)
SONY_FPGA_SLAVEDATA_FILE_WRITE = IOWR(SONY_FPGA_IOC_MAGIC, 25, slavedata_write_info)
SONY_FPGA_SLAVEDATA_UPDATE_PROGRESS = IOR(SONY_FPGA_IOC_MAGIC, 26, ctypes.c_uint)
SONY_FPGA_SLAVEDATA_UPDATE_START = IOW(SONY_FPGA_IOC_MAGIC, 27, slavedata_start_param)
SONY_FPGA_SLAVEDATA_UPDATE_FINISH = IOR(SONY_FPGA_IOC_MAGIC, 28, ctypes.c_uint)
SONY_FPGA_SLAVEDATA_UPDATE_ERROR_CLEAR = IOR(SONY_FPGA_IOC_MAGIC, 35, ctypes.c_uint)
SONY_FPGA_WAIT_VD = IO(SONY_FPGA_IOC_MAGIC, 29)
SONY_FPGA_READ_FRAME_CNT = IOR(SONY_FPGA_IOC_MAGIC, 30, ctypes.c_uint)
SONY_FPGA_CMP_TEST_CTRL = IOW(SONY_FPGA_IOC_MAGIC, 31, ctypes.c_uint)
SONY_FPGA_REFSYNC_MON_CLEAR = IO(SONY_FPGA_IOC_MAGIC, 32)
SONY_FPGA_SET_VIDEO_FORMAT = IOW(SONY_FPGA_IOC_MAGIC, 33, ctypes.c_uint)
SONY_FPGA_SET_VIDEO_TRANSFER = IOW(SONY_FPGA_IOC_MAGIC, 34, ctypes.c_uint)
SONY_FPGA_PAUSE_VIDEO_DMAC = IO(SONY_FPGA_IOC_MAGIC, 35)
SONY_FPGA_RESTART_VIDEO_DMAC = IO(SONY_FPGA_IOC_MAGIC, 36)
SONY_FPGA_SET_VIDEO_DMAC_CHANNEL = IOW(SONY_FPGA_IOC_MAGIC, 37, video_dmac_channel_setting)
